@@ -20,28 +20,14 @@
 #include <sys/stat.h>
 #include <sys/uio.h>
 #include <stdio.h>
-#ifdef STDC_HEADERS
-# include <stdlib.h>
-# include <stddef.h>
-#else
-# ifdef HAVE_STDLIB_H
-#  include <stdlib.h>
-# endif
-#endif /* STDC_HEADERS */
-#ifdef HAVE_STDBOOL_H
-# include <stdbool.h>
-#else
-# include "compat/stdbool.h"
-#endif
+#include <stdlib.h>
 #ifdef HAVE_STRING_H
 # include <string.h>
 #endif /* HAVE_STRING_H */
 #ifdef HAVE_STRINGS_H
 # include <strings.h>
 #endif /* HAVE_STRINGS_H */
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif /* HAVE_UNISTD_H */
+#include <unistd.h>
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -51,11 +37,10 @@
 #include "sudo_gettext.h"	/* must be included before sudo_compat.h */
 
 #include "sudo_compat.h"
-#include "sudo_alloc.h"
 #include "sudo_fatal.h"
 #include "sudo_plugin.h"
-#include "sudo_conf.h"
 #include "sudo_debug.h"
+#include "sudo_conf.h"
 #include "sudo_util.h"
 
 /*
@@ -139,29 +124,38 @@ static int sudo_debug_active_instance = -1;
 static void
 sudo_debug_free_output(struct sudo_debug_output *output)
 {
-    sudo_efree(output->filename);
-    sudo_efree(output->settings);
+    free(output->filename);
+    free(output->settings);
     if (output->fd != -1)
 	close(output->fd);
-    sudo_efree(output);
+    free(output);
 }
 
 /*
  * Create a new output file for the specified debug instance.
+ * Returns NULL if the file cannot be opened or memory cannot be allocated.
  */
 static struct sudo_debug_output *
 sudo_debug_new_output(struct sudo_debug_instance *instance,
     struct sudo_debug_file *debug_file)
 {
-    char *buf, *cp, *subsys, *pri;
+    char *buf, *cp, *last, *subsys, *pri;
     struct sudo_debug_output *output;
     unsigned int i, j;
 
     /* Create new output for the instance. */
     /* XXX - reuse fd for existing filename? */
-    output = sudo_emalloc(sizeof(*output));
-    output->settings = sudo_emallocarray(instance->max_subsystem + 1, sizeof(int));
-    output->filename = sudo_estrdup(debug_file->debug_file);
+    output = calloc(1, sizeof(*output));
+    if (output == NULL)
+	goto bad;
+    output->fd = -1;
+    output->settings = reallocarray(NULL, instance->max_subsystem + 1,
+	sizeof(int));
+    if (output->settings == NULL)
+	goto bad;
+    output->filename = strdup(debug_file->debug_file);
+    if (output->filename == NULL)
+	goto bad;
     output->fd = -1;
 
     /* Init per-subsystems settings to -1 since 0 is a valid priority. */
@@ -176,27 +170,33 @@ sudo_debug_new_output(struct sudo_debug_instance *instance,
 	    output->fd = open(output->filename, O_WRONLY|O_APPEND|O_CREAT,
 		S_IRUSR|S_IWUSR);
 	}
-	if (output->fd == -1) {
-	    sudo_debug_free_output(output);
-	    return NULL;
-	}
+	if (output->fd == -1)
+	    goto bad;
 	ignore_result(fchown(output->fd, (uid_t)-1, 0));
     }
     (void)fcntl(output->fd, F_SETFD, FD_CLOEXEC);
     if (sudo_debug_fds_size < output->fd) {
 	/* Bump fds size to the next multiple of 4 * NBBY. */
-	const int new_size = round_nfds(output->fd);
-	sudo_debug_fds = sudo_erecalloc(sudo_debug_fds,
-	    sudo_debug_fds_size / NBBY, new_size / NBBY, sizeof(char));
-	sudo_debug_fds_size = new_size;
+	const int old_size = sudo_debug_fds_size / NBBY;
+	const int new_size = round_nfds(output->fd + 1) / NBBY;
+	unsigned char *new_fds;
+
+	new_fds = realloc(sudo_debug_fds, new_size);
+	if (new_fds == NULL)
+	    goto bad;
+	memset(new_fds + old_size, 0, new_size - old_size);
+	sudo_debug_fds = new_fds;
+	sudo_debug_fds_size = new_size * NBBY;
     }
     sudo_setbit(sudo_debug_fds, output->fd);
     if (output->fd > sudo_debug_max_fd)
 	sudo_debug_max_fd = output->fd;
 
     /* Parse Debug conf string. */
-    buf = sudo_estrdup(debug_file->debug_flags);
-    for ((cp = strtok(buf, ",")); cp != NULL; (cp = strtok(NULL, ","))) {
+    buf = strdup(debug_file->debug_flags);
+    if (buf == NULL)
+	goto bad;
+    for ((cp = strtok_r(buf, ",", &last)); cp != NULL; (cp = strtok_r(NULL, ",", &last))) {
 	/* Should be in the form subsys@pri. */
 	subsys = cp;
 	if ((pri = strchr(cp, '@')) == NULL)
@@ -227,6 +227,11 @@ sudo_debug_new_output(struct sudo_debug_instance *instance,
     free(buf);
 
     return output;
+bad:
+    sudo_warn_nodebug(NULL);
+    if (output != NULL)
+	sudo_debug_free_output(output);
+    return NULL;
 }
 
 /*
@@ -246,6 +251,7 @@ sudo_debug_register_v1(const char *program, const char *const subsystems[],
     struct sudo_debug_output *output;
     struct sudo_debug_file *debug_file;
     int idx, free_idx = -1;
+    debug_decl_func(sudo_debug_register);
 
     if (debug_files == NULL)
 	return SUDO_DEBUG_INSTANCE_INITIALIZER;
@@ -299,8 +305,12 @@ sudo_debug_register_v1(const char *program, const char *const subsystems[],
 	    sudo_warnx_nodebug("%s: instance number mismatch: expected %d or %d, got %d", __func__, sudo_debug_last_instance + 1, free_idx, idx);
 	    return SUDO_DEBUG_INSTANCE_INITIALIZER;
 	}
-	instance = sudo_emalloc(sizeof(*instance));
-	instance->program = sudo_estrdup(program);
+	if ((instance = malloc(sizeof(*instance))) == NULL)
+	    return SUDO_DEBUG_INSTANCE_INITIALIZER;
+	if ((instance->program = strdup(program)) == NULL) {
+	    free(instance);
+	    return SUDO_DEBUG_INSTANCE_INITIALIZER;
+	}
 	instance->subsystems = subsystems;
 	instance->subsystem_ids = ids;
 	instance->max_subsystem = max_id;
@@ -346,6 +356,7 @@ sudo_debug_deregister_v1(int idx)
 {
     struct sudo_debug_instance *instance;
     struct sudo_debug_output *output, *next;
+    debug_decl_func(sudo_debug_deregister);
 
     if (idx < 0 || idx > sudo_debug_last_instance) {
 	sudo_warnx_nodebug("%s: invalid instance ID %d, max %d",
@@ -364,12 +375,12 @@ sudo_debug_deregister_v1(int idx)
     sudo_debug_instances[idx] = NULL;
     SLIST_FOREACH_SAFE(output, &instance->outputs, entries, next) {
 	close(output->fd);
-	sudo_efree(output->filename);
-	sudo_efree(output->settings);
-	sudo_efree(output);
+	free(output->filename);
+	free(output->settings);
+	free(output);
     }
-    sudo_efree(instance->program);
-    sudo_efree(instance);
+    free(instance->program);
+    free(instance);
 
     if (idx == sudo_debug_last_instance)
 	sudo_debug_last_instance--;
@@ -445,18 +456,12 @@ sudo_debug_exit_size_t_v1(const char *func, const char *file, int line,
 	"<- %s @ %s:%d := %zu", func, file, line, rval);
 }
 
-/* We use int, not bool, here for functions that return -1 on error. */
 void
 sudo_debug_exit_bool_v1(const char *func, const char *file, int line,
-    int subsys, int rval)
+    int subsys, bool rval)
 {
-    if (rval == true || rval == false) {
-	sudo_debug_printf2(NULL, NULL, 0, subsys | SUDO_DEBUG_TRACE,
-	    "<- %s @ %s:%d := %s", func, file, line, rval ? "true" : "false");
-    } else {
-	sudo_debug_printf2(NULL, NULL, 0, subsys | SUDO_DEBUG_TRACE,
-	    "<- %s @ %s:%d := %d", func, file, line, rval);
-    }
+    sudo_debug_printf2(NULL, NULL, 0, subsys | SUDO_DEBUG_TRACE,
+	"<- %s @ %s:%d := %s", func, file, line, rval ? "true" : "false");
 }
 
 void
@@ -572,6 +577,7 @@ sudo_debug_vprintf2_v1(const char *func, const char *file, int lineno, int level
     char static_buf[1024], *buf = static_buf;
     struct sudo_debug_instance *instance;
     struct sudo_debug_output *output;
+    debug_decl_func(sudo_debug_vprintf2);
 
     if (sudo_debug_active_instance == -1)
 	goto out;
@@ -617,7 +623,7 @@ sudo_debug_vprintf2_v1(const char *func, const char *file, int lineno, int level
 		else
 		    sudo_debug_write2(output->fd, NULL, NULL, 0, buf, buflen, errcode);
 		if (buf != static_buf) {
-		    sudo_efree(buf);
+		    free(buf);
 		    buf = static_buf;
 		}
 	    }
@@ -662,6 +668,7 @@ sudo_debug_execve2_v1(int level, const char *path, char *const argv[], char *con
     char * const *av;
     char *cp, static_buf[4096], *buf = static_buf;
     size_t plen;
+    debug_decl_func(sudo_debug_execve2);
 
     if (sudo_debug_active_instance == -1)
 	goto out;
@@ -752,7 +759,7 @@ sudo_debug_execve2_v1(int level, const char *path, char *const argv[], char *con
 
 	sudo_debug_write(output->fd, buf, buflen, 0);
 	if (buf != static_buf) {
-	    sudo_efree(buf);
+	    free(buf);
 	    buf = static_buf;
 	}
     }
