@@ -21,25 +21,18 @@
 
 #include <config.h>
 
+#ifdef HAVE_FWTK
+
 #include <sys/types.h>
 #include <stdio.h>
-#ifdef STDC_HEADERS
-# include <stdlib.h>
-# include <stddef.h>
-#else
-# ifdef HAVE_STDLIB_H
-#  include <stdlib.h>
-# endif
-#endif /* STDC_HEADERS */
+#include <stdlib.h>
 #ifdef HAVE_STRING_H
 # include <string.h>
 #endif /* HAVE_STRING_H */
 #ifdef HAVE_STRINGS_H
 # include <strings.h>
 #endif /* HAVE_STRING_H */
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif /* HAVE_UNISTD_H */
+#include <unistd.h>
 #include <pwd.h>
 
 #include <auth.h>
@@ -152,3 +145,5 @@ sudo_fwtk_cleanup(struct passwd *pw, sudo_auth *auth)
     auth_close();
     debug_return_int(AUTH_SUCCESS);
 }
+
+#endif /* HAVE_FWTK */
