@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999-2005, 2007, 2010-2012
+ * Copyright (c) 1999-2005, 2007, 2010-2012, 2014-2015
  *	Todd C. Miller <Todd.Miller@courtesan.com>
  * Copyright (c) 2002 Michael Stroucken <michael@stroucken.org>
  *
@@ -25,25 +25,18 @@
 
 #include <config.h>
 
+#ifdef HAVE_SECURID
+
 #include <sys/types.h>
 #include <stdio.h>
-#ifdef STDC_HEADERS
-# include <stdlib.h>
-# include <stddef.h>
-#else
-# ifdef HAVE_STDLIB_H
-#  include <stdlib.h>
-# endif
-#endif /* STDC_HEADERS */
+#include <stdlib.h>
 #ifdef HAVE_STRING_H
 # include <string.h>
 #endif /* HAVE_STRING_H */
 #ifdef HAVE_STRINGS_H
 # include <strings.h>
 #endif /* HAVE_STRINGS_H */
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif /* HAVE_UNISTD_H */
+#include <unistd.h>
 #include <pwd.h>
 
 /* Needed for SecurID v5.0 Authentication on UNIX */
@@ -69,7 +62,7 @@ int
 sudo_securid_init(struct passwd *pw, sudo_auth *auth)
 {
     static SDI_HANDLE sd_dat;			/* SecurID handle */
-    debug_decl(sudo_securid_init, SUDO_DEBUG_AUTH)
+    debug_decl(sudo_securid_init, SUDOERS_DEBUG_AUTH)
 
     auth->data = (void *) &sd_dat;		/* For method-specific data */
 
@@ -99,7 +92,7 @@ sudo_securid_setup(struct passwd *pw, char **promptp, sudo_auth *auth)
 {
     SDI_HANDLE *sd = (SDI_HANDLE *) auth->data;
     int retval;
-    debug_decl(sudo_securid_setup, SUDO_DEBUG_AUTH)
+    debug_decl(sudo_securid_setup, SUDOERS_DEBUG_AUTH)
 
     /* Re-initialize SecurID every time. */
     if (SD_Init(sd) != ACM_OK) {
@@ -150,7 +143,7 @@ sudo_securid_verify(struct passwd *pw, char *pass, sudo_auth *auth)
 {
     SDI_HANDLE *sd = (SDI_HANDLE *) auth->data;
     int rval;
-    debug_decl(sudo_securid_verify, SUDO_DEBUG_AUTH)
+    debug_decl(sudo_securid_verify, SUDOERS_DEBUG_AUTH)
 
     pass = auth_getpass("Enter your PASSCODE: ",
 	def_passwd_timeout * 60, SUDO_CONV_PROMPT_ECHO_OFF);
@@ -222,3 +215,5 @@ then enter the new token code.\n", \
     /* Return stored state to calling process */
     debug_return_int(rval);
 }
+
+#endif /* HAVE_SECURID */
