@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1993-1996,1998-2005, 2007-2014
+ * Copyright (c) 1993-1996,1998-2005, 2007-2015
  *	Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -24,23 +24,14 @@
 #include <sys/types.h>
 #include <sys/time.h>
 #include <stdio.h>
-#ifdef STDC_HEADERS
-# include <stdlib.h>
-# include <stddef.h>
-#else
-# ifdef HAVE_STDLIB_H
-#  include <stdlib.h>
-# endif
-#endif /* STDC_HEADERS */
+#include <stdlib.h>
 #ifdef HAVE_STRING_H
 # include <string.h>
 #endif /* HAVE_STRING_H */
 #ifdef HAVE_STRINGS_H
 # include <strings.h>
 #endif /* HAVE_STRINGS_H */
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif /* HAVE_UNISTD_H */
+#include <unistd.h>
 #ifdef TIME_WITH_SYS_TIME
 # include <time.h>
 #endif
@@ -62,29 +53,37 @@ static struct passwd *get_authpw(int);
 static int
 check_user_interactive(int validated, int mode, struct passwd *auth_pw)
 {
-    int status, rval = true;
-    debug_decl(check_user_interactive, SUDO_DEBUG_AUTH)
+    int status, rval = -1;
+    char *prompt;
+    bool lectured;
+    debug_decl(check_user_interactive, SUDOERS_DEBUG_AUTH)
 
     /* Always need a password when -k was specified with the command. */
     if (ISSET(mode, MODE_IGNORE_TICKET))
 	SET(validated, FLAG_CHECK_USER);
 
-    if (build_timestamp(auth_pw) == -1) {
-	rval = -1;
+    if (build_timestamp(auth_pw) == -1)
 	goto done;
-    }
 
     status = timestamp_status(auth_pw);
+    switch (status) {
+    case TS_FATAL:
+	/* Fatal error (usually setuid failure), unsafe to proceed. */
+	goto done;
 
-    if (status != TS_CURRENT || ISSET(validated, FLAG_CHECK_USER)) {
-	char *prompt;
-	bool lectured;
+    case TS_CURRENT:
+	/* Time stamp file is valid and current. */
+	if (!ISSET(validated, FLAG_CHECK_USER)) {
+	    rval = true;
+	    break;
+	}
+	/* FALLTHROUGH */
 
+    default:
 	/* Bail out if we are non-interactive and a password is required */
 	if (ISSET(mode, MODE_NONINTERACTIVE)) {
 	    validated |= FLAG_NON_INTERACTIVE;
 	    log_auth_failure(validated, 0);
-	    rval = -1;
 	    goto done;
 	}
 
@@ -94,22 +93,26 @@ check_user_interactive(int validated, int mode, struct passwd *auth_pw)
 	/* Expand any escapes in the prompt. */
 	prompt = expand_prompt(user_prompt ? user_prompt : def_passprompt,
 	    auth_pw->pw_name);
-	if (prompt == NULL) {
-	    rval = -1;
+	if (prompt == NULL)
 	    goto done;
-	}
 
 	rval = verify_user(auth_pw, prompt, validated);
-	if (rval == true && lectured)
-	    set_lectured();
-	sudo_efree(prompt);
+	if (rval == true && lectured) {
+	    if (set_lectured() == -1)
+		rval = -1;
+	}
+	free(prompt);
+	break;
     }
+
     /* Only update timestamp if user was validated. */
-    if (rval == true && ISSET(validated, VALIDATE_OK) &&
-	!ISSET(mode, MODE_IGNORE_TICKET) && status != TS_ERROR)
-	update_timestamp(auth_pw);
+    if (rval == true && ISSET(validated, VALIDATE_SUCCESS) &&
+	!ISSET(mode, MODE_IGNORE_TICKET) && status != TS_ERROR) {
+	if (update_timestamp(auth_pw) == -1)
+	    rval = -1;
+    }
 done:
-    debug_return_bool(rval);
+    debug_return_int(rval);
 }
 
 /*
@@ -121,7 +124,7 @@ check_user(int validated, int mode)
 {
     struct passwd *auth_pw;
     int rval = -1;
-    debug_decl(check_user, SUDO_DEBUG_AUTH)
+    debug_decl(check_user, SUDOERS_DEBUG_AUTH)
 
     /*
      * Init authentication system regardless of whether we need a password.
@@ -160,7 +163,7 @@ done:
     sudo_auth_cleanup(auth_pw);
     sudo_pw_delref(auth_pw);
 
-    debug_return_bool(rval);
+    debug_return_int(rval);
 }
 
 /*
@@ -175,7 +178,7 @@ display_lecture(int status)
     ssize_t nread;
     struct sudo_conv_message msg;
     struct sudo_conv_reply repl;
-    debug_decl(lecture, SUDO_DEBUG_AUTH)
+    debug_decl(lecture, SUDOERS_DEBUG_AUTH)
 
     if (def_lecture == never ||
 	(def_lecture == once && already_lectured(status)))
@@ -212,7 +215,7 @@ bool
 user_is_exempt(void)
 {
     bool rval = false;
-    debug_decl(user_is_exempt, SUDO_DEBUG_AUTH)
+    debug_decl(user_is_exempt, SUDOERS_DEBUG_AUTH)
 
     if (def_exempt_group)
 	rval = user_in_group(sudo_user.pw, def_exempt_group);
@@ -228,7 +231,7 @@ static struct passwd *
 get_authpw(int mode)
 {
     struct passwd *pw = NULL;
-    debug_decl(get_authpw, SUDO_DEBUG_AUTH)
+    debug_decl(get_authpw, SUDOERS_DEBUG_AUTH)
 
     if (ISSET(mode, (MODE_CHECK|MODE_LIST))) {
 	/* In list mode we always prompt for the user's password. */
