@@ -19,8 +19,8 @@
  * Materiel Command, USAF, under agreement number F39502-99-1-0512.
  */
 
-#ifndef _SUDOERS_CHECK_H
-#define _SUDOERS_CHECK_H
+#ifndef SUDOERS_CHECK_H
+#define SUDOERS_CHECK_H
 
 /* Status codes for timestamp_status() */
 #define TS_CURRENT		0
@@ -28,6 +28,7 @@
 #define TS_MISSING		2
 #define TS_NOFILE		3
 #define TS_ERROR		4
+#define TS_FATAL		5
 
 /*
  * Time stamps are now stored in a single file which contains multiple
@@ -61,8 +62,8 @@ struct timestamp_entry {
 };
 
 bool  already_lectured(int status);
-bool  update_timestamp(struct passwd *pw);
+int   update_timestamp(struct passwd *pw);
 int   build_timestamp(struct passwd *pw);
 int   timestamp_status(struct passwd *pw);
 
-#endif /* _SUDOERS_CHECK_H */
+#endif /* SUDOERS_CHECK_H */
