@@ -64,23 +64,15 @@
 
 #include <sys/types.h>
 #include <stdio.h>
-#ifdef STDC_HEADERS
-# include <stdlib.h>
-# include <stddef.h>
-#else
-# ifdef HAVE_STDLIB_H
-#  include <stdlib.h>
-# endif
-#endif /* STDC_HEADERS */
+#include <stdlib.h>
+#include <stddef.h>
 #ifdef HAVE_STRING_H
 # include <string.h>
 #endif /* HAVE_STRING_H */
 #ifdef HAVE_STRINGS_H
 # include <strings.h>
 #endif /* HAVE_STRINGS_H */
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif /* HAVE_UNISTD_H */
+#include <unistd.h>
 #if defined(HAVE_STDINT_H)
 # include <stdint.h>
 #elif defined(HAVE_INTTYPES_H)
@@ -109,12 +101,12 @@ struct userspec_list userspecs = TAILQ_HEAD_INITIALIZER(userspecs);
 /*
  * Local protoypes
  */
-static void  add_defaults(int, struct member *, struct defaults *);
-static void  add_userspec(struct member *, struct privilege *);
+static bool add_defaults(int, struct member *, struct defaults *);
+static bool add_userspec(struct member *, struct privilege *);
 static struct defaults *new_default(char *, char *, int);
 static struct member *new_member(char *, int);
 static struct sudo_digest *new_digest(int, const char *);
-#line 81 "gram.y"
+#line 73 "gram.y"
 #ifndef YYSTYPE_DEFINED
 #define YYSTYPE_DEFINED
 typedef union {
@@ -132,7 +124,7 @@ typedef union {
     int tok;
 } YYSTYPE;
 #endif /* YYSTYPE_DEFINED */
-#line 135 "gram.c"
+#line 127 "gram.c"
 #define COMMAND 257
 #define ALIAS 258
 #define DEFVAR 259
@@ -709,7 +701,7 @@ short *yyss;
 short *yysslim;
 YYSTYPE *yyvs;
 unsigned int yystacksize;
-#line 687 "gram.y"
+#line 850 "gram.y"
 void
 sudoerserror(const char *s)
 {
@@ -722,7 +714,7 @@ sudoerserror(const char *s)
     /* Save the line the first error occurred on. */
     if (errorlineno == -1) {
 	errorlineno = sudolineno;
-	errorfile = sudo_estrdup(sudoers);
+	errorfile = sudoers;
     }
     if (sudoers_warnings && s != NULL) {
 	LEXTRACE("<*> ");
@@ -748,7 +740,12 @@ new_default(char *var, char *val, int op)
     struct defaults *d;
     debug_decl(new_default, SUDOERS_DEBUG_PARSER)
 
-    d = sudo_ecalloc(1, sizeof(struct defaults));
+    if ((d = calloc(1, sizeof(struct defaults))) == NULL) {
+	sudo_debug_printf(SUDO_DEBUG_ERROR|SUDO_DEBUG_LINENO,
+	    "unable to allocate memory");
+	debug_return_ptr(NULL);
+    }
+
     d->var = var;
     d->val = val;
     /* d->type = 0; */
@@ -765,7 +762,12 @@ new_member(char *name, int type)
     struct member *m;
     debug_decl(new_member, SUDOERS_DEBUG_PARSER)
 
-    m = sudo_ecalloc(1, sizeof(struct member));
+    if ((m = calloc(1, sizeof(struct member))) == NULL) {
+	sudo_debug_printf(SUDO_DEBUG_ERROR|SUDO_DEBUG_LINENO,
+	    "unable to allocate memory");
+	debug_return_ptr(NULL);
+    }
+
     m->name = name;
     m->type = type;
     HLTQ_INIT(m, entries);
@@ -779,9 +781,20 @@ new_digest(int digest_type, const char *digest_str)
     struct sudo_digest *dig;
     debug_decl(new_digest, SUDOERS_DEBUG_PARSER)
 
-    dig = sudo_emalloc(sizeof(*dig));
+    if ((dig = malloc(sizeof(*dig))) == NULL) {
+	sudo_debug_printf(SUDO_DEBUG_ERROR|SUDO_DEBUG_LINENO,
+	    "unable to allocate memory");
+	debug_return_ptr(NULL);
+    }
+
     dig->digest_type = digest_type;
-    dig->digest_str = sudo_estrdup(digest_str);
+    dig->digest_str = strdup(digest_str);
+    if (dig->digest_str == NULL) {
+	sudo_debug_printf(SUDO_DEBUG_ERROR|SUDO_DEBUG_LINENO,
+	    "unable to allocate memory");
+	free(dig);
+	dig = NULL;
+    }
 
     debug_return_ptr(dig);
 }
@@ -791,7 +804,7 @@ new_digest(int digest_type, const char *digest_str)
  * The binding, if non-NULL, specifies a list of hosts, users, or
  * runas users the entries apply to (specified by the type).
  */
-static void
+static bool
 add_defaults(int type, struct member *bmem, struct defaults *defs)
 {
     struct defaults *d;
@@ -802,7 +815,11 @@ add_defaults(int type, struct member *bmem, struct defaults *defs)
 	/*
 	 * We use a single binding for each entry in defs.
 	 */
-	binding = sudo_emalloc(sizeof(*binding));
+	if ((binding = malloc(sizeof(*binding))) == NULL) {
+	    sudo_debug_printf(SUDO_DEBUG_ERROR|SUDO_DEBUG_LINENO,
+		"unable to allocate memory");
+	    debug_return_bool(false);
+	}
 	if (bmem != NULL)
 	    HLTQ_TO_TAILQ(binding, bmem, entries);
 	else
@@ -819,37 +836,42 @@ add_defaults(int type, struct member *bmem, struct defaults *defs)
 	TAILQ_CONCAT_HLTQ(&defaults, defs, entries);
     }
 
-    debug_return;
+    debug_return_bool(true);
 }
 
 /*
  * Allocate a new struct userspec, populate it, and insert it at the
  * end of the userspecs list.
  */
-static void
+static bool
 add_userspec(struct member *members, struct privilege *privs)
 {
     struct userspec *u;
     debug_decl(add_userspec, SUDOERS_DEBUG_PARSER)
 
-    u = sudo_ecalloc(1, sizeof(*u));
+    if ((u = calloc(1, sizeof(*u))) == NULL) {
+	sudo_debug_printf(SUDO_DEBUG_ERROR|SUDO_DEBUG_LINENO,
+	    "unable to allocate memory");
+	debug_return_bool(false);
+    }
     HLTQ_TO_TAILQ(&u->users, members, entries);
     HLTQ_TO_TAILQ(&u->privileges, privs, entries);
     TAILQ_INSERT_TAIL(&userspecs, u, entries);
 
-    debug_return;
+    debug_return_bool(true);
 }
 
 /*
  * Free up space used by data structures from a previous parser run and sets
  * the current sudoers file to path.
  */
-void
+bool
 init_parser(const char *path, bool quiet)
 {
     struct member_list *binding;
     struct defaults *d, *d_next;
     struct userspec *us, *us_next;
+    bool rval = true;
     debug_decl(init_parser, SUDOERS_DEBUG_PARSER)
 
     TAILQ_FOREACH_SAFE(us, &userspecs, entries, us_next) {
@@ -857,8 +879,8 @@ init_parser(const char *path, bool quiet)
 	struct privilege *priv, *priv_next;
 
 	TAILQ_FOREACH_SAFE(m, &us->users, entries, m_next) {
-	    sudo_efree(m->name);
-	    sudo_efree(m);
+	    free(m->name);
+	    free(m);
 	}
 	TAILQ_FOREACH_SAFE(priv, &us->privileges, entries, priv_next) {
 	    struct member_list *runasuserlist = NULL, *runasgrouplist = NULL;
@@ -871,62 +893,62 @@ init_parser(const char *path, bool quiet)
 #endif /* HAVE_PRIV_SET */
 
 	    TAILQ_FOREACH_SAFE(m, &priv->hostlist, entries, m_next) {
-		sudo_efree(m->name);
-		sudo_efree(m);
+		free(m->name);
+		free(m);
 	    }
 	    TAILQ_FOREACH_SAFE(cs, &priv->cmndlist, entries, cs_next) {
 #ifdef HAVE_SELINUX
 		/* Only free the first instance of a role/type. */
 		if (cs->role != role) {
 		    role = cs->role;
-		    sudo_efree(cs->role);
+		    free(cs->role);
 		}
 		if (cs->type != type) {
 		    type = cs->type;
-		    sudo_efree(cs->type);
+		    free(cs->type);
 		}
 #endif /* HAVE_SELINUX */
 #ifdef HAVE_PRIV_SET
 		/* Only free the first instance of privs/limitprivs. */
 		if (cs->privs != privs) {
 		    privs = cs->privs;
-		    sudo_efree(cs->privs);
+		    free(cs->privs);
 		}
 		if (cs->limitprivs != limitprivs) {
 		    limitprivs = cs->limitprivs;
-		    sudo_efree(cs->limitprivs);
+		    free(cs->limitprivs);
 		}
 #endif /* HAVE_PRIV_SET */
 		/* Only free the first instance of runas user/group lists. */
 		if (cs->runasuserlist && cs->runasuserlist != runasuserlist) {
 		    runasuserlist = cs->runasuserlist;
 		    TAILQ_FOREACH_SAFE(m, runasuserlist, entries, m_next) {
-			sudo_efree(m->name);
-			sudo_efree(m);
+			free(m->name);
+			free(m);
 		    }
-		    sudo_efree(runasuserlist);
+		    free(runasuserlist);
 		}
 		if (cs->runasgrouplist && cs->runasgrouplist != runasgrouplist) {
 		    runasgrouplist = cs->runasgrouplist;
 		    TAILQ_FOREACH_SAFE(m, runasgrouplist, entries, m_next) {
-			sudo_efree(m->name);
-			sudo_efree(m);
+			free(m->name);
+			free(m);
 		    }
-		    sudo_efree(runasgrouplist);
+		    free(runasgrouplist);
 		}
 		if (cs->cmnd->type == COMMAND) {
 			struct sudo_command *c =
 			    (struct sudo_command *) cs->cmnd->name;
-			sudo_efree(c->cmnd);
-			sudo_efree(c->args);
+			free(c->cmnd);
+			free(c->args);
 		}
-		sudo_efree(cs->cmnd->name);
-		sudo_efree(cs->cmnd);
-		sudo_efree(cs);
+		free(cs->cmnd->name);
+		free(cs->cmnd);
+		free(cs);
 	    }
-	    sudo_efree(priv);
+	    free(priv);
 	}
-	sudo_efree(us);
+	free(us);
     }
     TAILQ_INIT(&userspecs);
 
@@ -940,35 +962,45 @@ init_parser(const char *path, bool quiet)
 		if (m->type == COMMAND) {
 			struct sudo_command *c =
 			    (struct sudo_command *) m->name;
-			sudo_efree(c->cmnd);
-			sudo_efree(c->args);
+			free(c->cmnd);
+			free(c->args);
 		}
-		sudo_efree(m->name);
-		sudo_efree(m);
+		free(m->name);
+		free(m);
 	    }
-	    sudo_efree(d->binding);
+	    free(d->binding);
 	}
-	sudo_efree(d->var);
-	sudo_efree(d->val);
-	sudo_efree(d);
+	free(d->var);
+	free(d->val);
+	free(d);
     }
     TAILQ_INIT(&defaults);
 
-    init_aliases();
-
     init_lexer();
 
-    sudo_efree(sudoers);
-    sudoers = path ? sudo_estrdup(path) : NULL;
+    if (!init_aliases()) {
+	sudo_warnx(U_("%s: %s"), __func__, U_("unable to allocate memory"));
+	rval = false;
+    }
+
+    free(sudoers);
+    if (path != NULL) {
+	if ((sudoers = strdup(path)) == NULL) {
+	    sudo_warnx(U_("%s: %s"), __func__, U_("unable to allocate memory"));
+	    rval = false;
+	}
+    } else {
+	sudoers = NULL;
+    }
 
     parse_error = false;
     errorlineno = -1;
     errorfile = sudoers;
     sudoers_warnings = !quiet;
 
-    debug_return;
+    debug_return_bool(rval);
 }
-#line 919 "gram.c"
+#line 951 "gram.c"
 /* allocate initial stack or double stack size, up to YYMAXDEPTH */
 #if defined(__cplusplus) || defined(__STDC__)
 static int yygrowstack(void)
@@ -1177,129 +1209,171 @@ yyreduce:
     switch (yyn)
     {
 case 1:
-#line 173 "gram.y"
+#line 165 "gram.y"
 { ; }
 break;
 case 5:
-#line 181 "gram.y"
+#line 173 "gram.y"
 {
 			    ;
 			}
 break;
 case 6:
-#line 184 "gram.y"
+#line 176 "gram.y"
 {
 			    yyerrok;
 			}
 break;
 case 7:
-#line 187 "gram.y"
+#line 179 "gram.y"
 {
-			    add_userspec(yyvsp[-1].member, yyvsp[0].privilege);
+			    if (!add_userspec(yyvsp[-1].member, yyvsp[0].privilege)) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 8:
-#line 190 "gram.y"
+#line 185 "gram.y"
 {
 			    ;
 			}
 break;
 case 9:
-#line 193 "gram.y"
+#line 188 "gram.y"
 {
 			    ;
 			}
 break;
 case 10:
-#line 196 "gram.y"
+#line 191 "gram.y"
 {
 			    ;
 			}
 break;
 case 11:
-#line 199 "gram.y"
+#line 194 "gram.y"
 {
 			    ;
 			}
 break;
 case 12:
-#line 202 "gram.y"
+#line 197 "gram.y"
 {
-			    add_defaults(DEFAULTS, NULL, yyvsp[0].defaults);
+			    if (!add_defaults(DEFAULTS, NULL, yyvsp[0].defaults)) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 13:
-#line 205 "gram.y"
+#line 203 "gram.y"
 {
-			    add_defaults(DEFAULTS_USER, yyvsp[-1].member, yyvsp[0].defaults);
+			    if (!add_defaults(DEFAULTS_USER, yyvsp[-1].member, yyvsp[0].defaults)) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 14:
-#line 208 "gram.y"
+#line 209 "gram.y"
 {
-			    add_defaults(DEFAULTS_RUNAS, yyvsp[-1].member, yyvsp[0].defaults);
+			    if (!add_defaults(DEFAULTS_RUNAS, yyvsp[-1].member, yyvsp[0].defaults)) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 15:
-#line 211 "gram.y"
+#line 215 "gram.y"
 {
-			    add_defaults(DEFAULTS_HOST, yyvsp[-1].member, yyvsp[0].defaults);
+			    if (!add_defaults(DEFAULTS_HOST, yyvsp[-1].member, yyvsp[0].defaults)) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 16:
-#line 214 "gram.y"
+#line 221 "gram.y"
 {
-			    add_defaults(DEFAULTS_CMND, yyvsp[-1].member, yyvsp[0].defaults);
+			    if (!add_defaults(DEFAULTS_CMND, yyvsp[-1].member, yyvsp[0].defaults)) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 18:
-#line 220 "gram.y"
+#line 230 "gram.y"
 {
 			    HLTQ_CONCAT(yyvsp[-2].defaults, yyvsp[0].defaults, entries);
 			    yyval.defaults = yyvsp[-2].defaults;
 			}
 break;
 case 19:
-#line 226 "gram.y"
+#line 236 "gram.y"
 {
 			    yyval.defaults = new_default(yyvsp[0].string, NULL, true);
+			    if (yyval.defaults == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 20:
-#line 229 "gram.y"
+#line 243 "gram.y"
 {
 			    yyval.defaults = new_default(yyvsp[0].string, NULL, false);
+			    if (yyval.defaults == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 21:
-#line 232 "gram.y"
+#line 250 "gram.y"
 {
 			    yyval.defaults = new_default(yyvsp[-2].string, yyvsp[0].string, true);
+			    if (yyval.defaults == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 22:
-#line 235 "gram.y"
+#line 257 "gram.y"
 {
 			    yyval.defaults = new_default(yyvsp[-2].string, yyvsp[0].string, '+');
+			    if (yyval.defaults == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 23:
-#line 238 "gram.y"
+#line 264 "gram.y"
 {
 			    yyval.defaults = new_default(yyvsp[-2].string, yyvsp[0].string, '-');
+			    if (yyval.defaults == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 25:
-#line 244 "gram.y"
+#line 274 "gram.y"
 {
 			    HLTQ_CONCAT(yyvsp[-2].privilege, yyvsp[0].privilege, entries);
 			    yyval.privilege = yyvsp[-2].privilege;
 			}
 break;
 case 26:
-#line 250 "gram.y"
+#line 280 "gram.y"
 {
-			    struct privilege *p = sudo_ecalloc(1, sizeof(*p));
+			    struct privilege *p = calloc(1, sizeof(*p));
+			    if (p == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			    HLTQ_TO_TAILQ(&p->hostlist, yyvsp[-2].member, entries);
 			    HLTQ_TO_TAILQ(&p->cmndlist, yyvsp[0].cmndspec, entries);
 			    HLTQ_INIT(p, entries);
@@ -1307,51 +1381,71 @@ case 26:
 			}
 break;
 case 27:
-#line 259 "gram.y"
+#line 293 "gram.y"
 {
 			    yyval.member = yyvsp[0].member;
 			    yyval.member->negated = false;
 			}
 break;
 case 28:
-#line 263 "gram.y"
+#line 297 "gram.y"
 {
 			    yyval.member = yyvsp[0].member;
 			    yyval.member->negated = true;
 			}
 break;
 case 29:
-#line 269 "gram.y"
+#line 303 "gram.y"
 {
 			    yyval.member = new_member(yyvsp[0].string, ALIAS);
+			    if (yyval.member == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 30:
-#line 272 "gram.y"
+#line 310 "gram.y"
 {
 			    yyval.member = new_member(NULL, ALL);
+			    if (yyval.member == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 31:
-#line 275 "gram.y"
+#line 317 "gram.y"
 {
 			    yyval.member = new_member(yyvsp[0].string, NETGROUP);
+			    if (yyval.member == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 32:
-#line 278 "gram.y"
+#line 324 "gram.y"
 {
 			    yyval.member = new_member(yyvsp[0].string, NTWKADDR);
+			    if (yyval.member == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 33:
-#line 281 "gram.y"
+#line 331 "gram.y"
 {
 			    yyval.member = new_member(yyvsp[0].string, WORD);
+			    if (yyval.member == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 35:
-#line 287 "gram.y"
+#line 341 "gram.y"
 {
 			    struct cmndspec *prev;
 			    prev = HLTQ_LAST(yyvsp[-2].cmndspec, cmndspec, entries);
@@ -1395,23 +1489,35 @@ case 35:
 			}
 break;
 case 36:
-#line 330 "gram.y"
+#line 384 "gram.y"
 {
-			    struct cmndspec *cs = sudo_ecalloc(1, sizeof(*cs));
+			    struct cmndspec *cs = calloc(1, sizeof(*cs));
+			    if (cs == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			    if (yyvsp[-4].runas != NULL) {
 				if (yyvsp[-4].runas->runasusers != NULL) {
 				    cs->runasuserlist =
-					sudo_emalloc(sizeof(*cs->runasuserlist));
+					malloc(sizeof(*cs->runasuserlist));
+				    if (cs->runasuserlist == NULL) {
+					sudoerserror(N_("unable to allocate memory"));
+					YYERROR;
+				    }
 				    HLTQ_TO_TAILQ(cs->runasuserlist,
 					yyvsp[-4].runas->runasusers, entries);
 				}
 				if (yyvsp[-4].runas->runasgroups != NULL) {
 				    cs->runasgrouplist =
-					sudo_emalloc(sizeof(*cs->runasgrouplist));
+					malloc(sizeof(*cs->runasgrouplist));
+				    if (cs->runasgrouplist == NULL) {
+					sudoerserror(N_("unable to allocate memory"));
+					YYERROR;
+				    }
 				    HLTQ_TO_TAILQ(cs->runasgrouplist,
 					yyvsp[-4].runas->runasgroups, entries);
 				}
-				sudo_efree(yyvsp[-4].runas);
+				free(yyvsp[-4].runas);
 			    }
 #ifdef HAVE_SELINUX
 			    cs->role = yyvsp[-3].seinfo.role;
@@ -1432,37 +1538,53 @@ case 36:
 			}
 break;
 case 37:
-#line 366 "gram.y"
+#line 432 "gram.y"
 {
 			    yyval.digest = new_digest(SUDO_DIGEST_SHA224, yyvsp[0].string);
+			    if (yyval.digest == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 38:
-#line 369 "gram.y"
+#line 439 "gram.y"
 {
 			    yyval.digest = new_digest(SUDO_DIGEST_SHA256, yyvsp[0].string);
+			    if (yyval.digest == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 39:
-#line 372 "gram.y"
+#line 446 "gram.y"
 {
 			    yyval.digest = new_digest(SUDO_DIGEST_SHA384, yyvsp[0].string);
+			    if (yyval.digest == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 40:
-#line 375 "gram.y"
+#line 453 "gram.y"
 {
 			    yyval.digest = new_digest(SUDO_DIGEST_SHA512, yyvsp[0].string);
+			    if (yyval.digest == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 41:
-#line 380 "gram.y"
+#line 462 "gram.y"
 {
 			    yyval.member = yyvsp[0].member;
 			}
 break;
 case 42:
-#line 383 "gram.y"
+#line 465 "gram.y"
 {
 			    if (yyvsp[0].member->type != COMMAND) {
 				sudoerserror(N_("a digest requires a path name"));
@@ -1474,269 +1596,318 @@ case 42:
 			}
 break;
 case 43:
-#line 394 "gram.y"
+#line 476 "gram.y"
 {
 			    yyval.member = yyvsp[0].member;
 			    yyval.member->negated = false;
 			}
 break;
 case 44:
-#line 398 "gram.y"
+#line 480 "gram.y"
 {
 			    yyval.member = yyvsp[0].member;
 			    yyval.member->negated = true;
 			}
 break;
 case 45:
-#line 404 "gram.y"
+#line 486 "gram.y"
 {
 			    yyval.string = yyvsp[0].string;
 			}
 break;
 case 46:
-#line 409 "gram.y"
+#line 491 "gram.y"
 {
 			    yyval.string = yyvsp[0].string;
 			}
 break;
 case 47:
-#line 414 "gram.y"
+#line 496 "gram.y"
 {
 			    yyval.seinfo.role = NULL;
 			    yyval.seinfo.type = NULL;
 			}
 break;
 case 48:
-#line 418 "gram.y"
+#line 500 "gram.y"
 {
 			    yyval.seinfo.role = yyvsp[0].string;
 			    yyval.seinfo.type = NULL;
 			}
 break;
 case 49:
-#line 422 "gram.y"
+#line 504 "gram.y"
 {
 			    yyval.seinfo.type = yyvsp[0].string;
 			    yyval.seinfo.role = NULL;
 			}
 break;
 case 50:
-#line 426 "gram.y"
+#line 508 "gram.y"
 {
 			    yyval.seinfo.role = yyvsp[-1].string;
 			    yyval.seinfo.type = yyvsp[0].string;
 			}
 break;
 case 51:
-#line 430 "gram.y"
+#line 512 "gram.y"
 {
 			    yyval.seinfo.type = yyvsp[-1].string;
 			    yyval.seinfo.role = yyvsp[0].string;
 			}
 break;
 case 52:
-#line 436 "gram.y"
+#line 518 "gram.y"
 {
 			    yyval.string = yyvsp[0].string;
 			}
 break;
 case 53:
-#line 440 "gram.y"
+#line 522 "gram.y"
 {
 			    yyval.string = yyvsp[0].string;
 			}
 break;
 case 54:
-#line 445 "gram.y"
+#line 527 "gram.y"
 {
 			    yyval.privinfo.privs = NULL;
 			    yyval.privinfo.limitprivs = NULL;
 			}
 break;
 case 55:
-#line 449 "gram.y"
+#line 531 "gram.y"
 {
 			    yyval.privinfo.privs = yyvsp[0].string;
 			    yyval.privinfo.limitprivs = NULL;
 			}
 break;
 case 56:
-#line 453 "gram.y"
+#line 535 "gram.y"
 {
 			    yyval.privinfo.privs = NULL;
 			    yyval.privinfo.limitprivs = yyvsp[0].string;
 			}
 break;
 case 57:
-#line 457 "gram.y"
+#line 539 "gram.y"
 {
 			    yyval.privinfo.privs = yyvsp[-1].string;
 			    yyval.privinfo.limitprivs = yyvsp[0].string;
 			}
 break;
 case 58:
-#line 461 "gram.y"
+#line 543 "gram.y"
 {
 			    yyval.privinfo.limitprivs = yyvsp[-1].string;
 			    yyval.privinfo.privs = yyvsp[0].string;
 			}
 break;
 case 59:
-#line 467 "gram.y"
+#line 549 "gram.y"
 {
 			    yyval.runas = NULL;
 			}
 break;
 case 60:
-#line 470 "gram.y"
+#line 552 "gram.y"
 {
 			    yyval.runas = yyvsp[-1].runas;
 			}
 break;
 case 61:
-#line 475 "gram.y"
+#line 557 "gram.y"
 {
-			    yyval.runas = sudo_ecalloc(1, sizeof(struct runascontainer));
-			    yyval.runas->runasusers = new_member(NULL, MYSELF);
-			    /* $$->runasgroups = NULL; */
+			    yyval.runas = calloc(1, sizeof(struct runascontainer));
+			    if (yyval.runas != NULL) {
+				yyval.runas->runasusers = new_member(NULL, MYSELF);
+				/* $$->runasgroups = NULL; */
+				if (yyval.runas->runasusers == NULL) {
+				    free(yyval.runas);
+				    yyval.runas = NULL;
+				}
+			    }
+			    if (yyval.runas == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 62:
-#line 480 "gram.y"
+#line 572 "gram.y"
 {
-			    yyval.runas = sudo_ecalloc(1, sizeof(struct runascontainer));
+			    yyval.runas = calloc(1, sizeof(struct runascontainer));
+			    if (yyval.runas == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			    yyval.runas->runasusers = yyvsp[0].member;
 			    /* $$->runasgroups = NULL; */
 			}
 break;
 case 63:
-#line 485 "gram.y"
+#line 581 "gram.y"
 {
-			    yyval.runas = sudo_ecalloc(1, sizeof(struct runascontainer));
+			    yyval.runas = calloc(1, sizeof(struct runascontainer));
+			    if (yyval.runas == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			    yyval.runas->runasusers = yyvsp[-2].member;
 			    yyval.runas->runasgroups = yyvsp[0].member;
 			}
 break;
 case 64:
-#line 490 "gram.y"
+#line 590 "gram.y"
 {
-			    yyval.runas = sudo_ecalloc(1, sizeof(struct runascontainer));
+			    yyval.runas = calloc(1, sizeof(struct runascontainer));
+			    if (yyval.runas == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			    /* $$->runasusers = NULL; */
 			    yyval.runas->runasgroups = yyvsp[0].member;
 			}
 break;
 case 65:
-#line 495 "gram.y"
+#line 599 "gram.y"
 {
-			    yyval.runas = sudo_ecalloc(1, sizeof(struct runascontainer));
-			    yyval.runas->runasusers = new_member(NULL, MYSELF);
-			    /* $$->runasgroups = NULL; */
+			    yyval.runas = calloc(1, sizeof(struct runascontainer));
+			    if (yyval.runas != NULL) {
+				yyval.runas->runasusers = new_member(NULL, MYSELF);
+				/* $$->runasgroups = NULL; */
+				if (yyval.runas->runasusers == NULL) {
+				    free(yyval.runas);
+				    yyval.runas = NULL;
+				}
+			    }
+			    if (yyval.runas == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 66:
-#line 502 "gram.y"
+#line 616 "gram.y"
 {
 			    yyval.tag.log_input = yyval.tag.log_output = yyval.tag.noexec =
 				yyval.tag.nopasswd = yyval.tag.send_mail = yyval.tag.setenv = UNSPEC;
 			}
 break;
 case 67:
-#line 506 "gram.y"
+#line 620 "gram.y"
 {
 			    yyval.tag.nopasswd = true;
 			}
 break;
 case 68:
-#line 509 "gram.y"
+#line 623 "gram.y"
 {
 			    yyval.tag.nopasswd = false;
 			}
 break;
 case 69:
-#line 512 "gram.y"
+#line 626 "gram.y"
 {
 			    yyval.tag.noexec = true;
 			}
 break;
 case 70:
-#line 515 "gram.y"
+#line 629 "gram.y"
 {
 			    yyval.tag.noexec = false;
 			}
 break;
 case 71:
-#line 518 "gram.y"
+#line 632 "gram.y"
 {
 			    yyval.tag.setenv = true;
 			}
 break;
 case 72:
-#line 521 "gram.y"
+#line 635 "gram.y"
 {
 			    yyval.tag.setenv = false;
 			}
 break;
 case 73:
-#line 524 "gram.y"
+#line 638 "gram.y"
 {
 			    yyval.tag.log_input = true;
 			}
 break;
 case 74:
-#line 527 "gram.y"
+#line 641 "gram.y"
 {
 			    yyval.tag.log_input = false;
 			}
 break;
 case 75:
-#line 530 "gram.y"
+#line 644 "gram.y"
 {
 			    yyval.tag.log_output = true;
 			}
 break;
 case 76:
-#line 533 "gram.y"
+#line 647 "gram.y"
 {
 			    yyval.tag.log_output = false;
 			}
 break;
 case 77:
-#line 536 "gram.y"
+#line 650 "gram.y"
 {
 			    yyval.tag.send_mail = true;
 			}
 break;
 case 78:
-#line 539 "gram.y"
+#line 653 "gram.y"
 {
 			    yyval.tag.send_mail = false;
 			}
 break;
 case 79:
-#line 544 "gram.y"
+#line 658 "gram.y"
 {
 			    yyval.member = new_member(NULL, ALL);
+			    if (yyval.member == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 80:
-#line 547 "gram.y"
+#line 665 "gram.y"
 {
 			    yyval.member = new_member(yyvsp[0].string, ALIAS);
+			    if (yyval.member == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 81:
-#line 550 "gram.y"
+#line 672 "gram.y"
 {
-			    struct sudo_command *c = sudo_ecalloc(1, sizeof(*c));
+			    struct sudo_command *c = calloc(1, sizeof(*c));
+			    if (c == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			    c->cmnd = yyvsp[0].command.cmnd;
 			    c->args = yyvsp[0].command.args;
 			    yyval.member = new_member((char *)c, COMMAND);
+			    if (yyval.member == NULL) {
+				free(c);
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 84:
-#line 562 "gram.y"
+#line 693 "gram.y"
 {
-			    char *s;
+			    const char *s;
 			    if ((s = alias_add(yyvsp[-2].string, HOSTALIAS, yyvsp[0].member)) != NULL) {
 				sudoerserror(s);
 				YYERROR;
@@ -1744,16 +1915,16 @@ case 84:
 			}
 break;
 case 86:
-#line 572 "gram.y"
+#line 703 "gram.y"
 {
 			    HLTQ_CONCAT(yyvsp[-2].member, yyvsp[0].member, entries);
 			    yyval.member = yyvsp[-2].member;
 			}
 break;
 case 89:
-#line 582 "gram.y"
+#line 713 "gram.y"
 {
-			    char *s;
+			    const char *s;
 			    if ((s = alias_add(yyvsp[-2].string, CMNDALIAS, yyvsp[0].member)) != NULL) {
 				sudoerserror(s);
 				YYERROR;
@@ -1761,16 +1932,16 @@ case 89:
 			}
 break;
 case 91:
-#line 592 "gram.y"
+#line 723 "gram.y"
 {
 			    HLTQ_CONCAT(yyvsp[-2].member, yyvsp[0].member, entries);
 			    yyval.member = yyvsp[-2].member;
 			}
 break;
 case 94:
-#line 602 "gram.y"
+#line 733 "gram.y"
 {
-			    char *s;
+			    const char *s;
 			    if ((s = alias_add(yyvsp[-2].string, RUNASALIAS, yyvsp[0].member)) != NULL) {
 				sudoerserror(s);
 				YYERROR;
@@ -1778,9 +1949,9 @@ case 94:
 			}
 break;
 case 97:
-#line 615 "gram.y"
+#line 746 "gram.y"
 {
-			    char *s;
+			    const char *s;
 			    if ((s = alias_add(yyvsp[-2].string, USERALIAS, yyvsp[0].member)) != NULL) {
 				sudoerserror(s);
 				YYERROR;
@@ -1788,96 +1959,128 @@ case 97:
 			}
 break;
 case 99:
-#line 625 "gram.y"
+#line 756 "gram.y"
 {
 			    HLTQ_CONCAT(yyvsp[-2].member, yyvsp[0].member, entries);
 			    yyval.member = yyvsp[-2].member;
 			}
 break;
 case 100:
-#line 631 "gram.y"
+#line 762 "gram.y"
 {
 			    yyval.member = yyvsp[0].member;
 			    yyval.member->negated = false;
 			}
 break;
 case 101:
-#line 635 "gram.y"
+#line 766 "gram.y"
 {
 			    yyval.member = yyvsp[0].member;
 			    yyval.member->negated = true;
 			}
 break;
 case 102:
-#line 641 "gram.y"
+#line 772 "gram.y"
 {
 			    yyval.member = new_member(yyvsp[0].string, ALIAS);
+			    if (yyval.member == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 103:
-#line 644 "gram.y"
+#line 779 "gram.y"
 {
 			    yyval.member = new_member(NULL, ALL);
+			    if (yyval.member == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 104:
-#line 647 "gram.y"
+#line 786 "gram.y"
 {
 			    yyval.member = new_member(yyvsp[0].string, NETGROUP);
+			    if (yyval.member == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 105:
-#line 650 "gram.y"
+#line 793 "gram.y"
 {
 			    yyval.member = new_member(yyvsp[0].string, USERGROUP);
+			    if (yyval.member == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 106:
-#line 653 "gram.y"
+#line 800 "gram.y"
 {
 			    yyval.member = new_member(yyvsp[0].string, WORD);
+			    if (yyval.member == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 108:
-#line 659 "gram.y"
+#line 810 "gram.y"
 {
 			    HLTQ_CONCAT(yyvsp[-2].member, yyvsp[0].member, entries);
 			    yyval.member = yyvsp[-2].member;
 			}
 break;
 case 109:
-#line 665 "gram.y"
+#line 816 "gram.y"
 {
 			    yyval.member = yyvsp[0].member;
 			    yyval.member->negated = false;
 			}
 break;
 case 110:
-#line 669 "gram.y"
+#line 820 "gram.y"
 {
 			    yyval.member = yyvsp[0].member;
 			    yyval.member->negated = true;
 			}
 break;
 case 111:
-#line 675 "gram.y"
+#line 826 "gram.y"
 {
 			    yyval.member = new_member(yyvsp[0].string, ALIAS);
+			    if (yyval.member == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 112:
-#line 678 "gram.y"
+#line 833 "gram.y"
 {
 			    yyval.member = new_member(NULL, ALL);
+			    if (yyval.member == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
 case 113:
-#line 681 "gram.y"
+#line 840 "gram.y"
 {
 			    yyval.member = new_member(yyvsp[0].string, WORD);
+			    if (yyval.member == NULL) {
+				sudoerserror(N_("unable to allocate memory"));
+				YYERROR;
+			    }
 			}
 break;
-#line 1828 "gram.c"
+#line 2031 "gram.c"
     }
     yyssp -= yym;
     yystate = *yyssp;
