@@ -21,22 +21,24 @@
 #define NOLOG_INPUT 277
 #define LOG_OUTPUT 278
 #define NOLOG_OUTPUT 279
-#define ALL 280
-#define COMMENT 281
-#define HOSTALIAS 282
-#define CMNDALIAS 283
-#define USERALIAS 284
-#define RUNASALIAS 285
-#define ERROR 286
-#define TYPE 287
-#define ROLE 288
-#define PRIVS 289
-#define LIMITPRIVS 290
-#define MYSELF 291
-#define SHA224_TOK 292
-#define SHA256_TOK 293
-#define SHA384_TOK 294
-#define SHA512_TOK 295
+#define MAIL 280
+#define NOMAIL 281
+#define ALL 282
+#define COMMENT 283
+#define HOSTALIAS 284
+#define CMNDALIAS 285
+#define USERALIAS 286
+#define RUNASALIAS 287
+#define ERROR 288
+#define TYPE 289
+#define ROLE 290
+#define PRIVS 291
+#define LIMITPRIVS 292
+#define MYSELF 293
+#define SHA224_TOK 294
+#define SHA256_TOK 295
+#define SHA384_TOK 296
+#define SHA512_TOK 297
 #ifndef YYSTYPE_DEFINED
 #define YYSTYPE_DEFINED
 typedef union {
