@@ -16,34 +16,15 @@
 
 #include <config.h>
 
-#include <sys/param.h>		/* for howmany() on Linux */
-#ifdef HAVE_SYS_SYSMACROS_H
-# include <sys/sysmacros.h>	/* for howmany() on Solaris */
-#endif /* HAVE_SYS_SYSMACROS_H */
-#ifdef HAVE_SYS_SELECT_H
-# include <sys/select.h>	/* for FD_* macros */
-#endif /* HAVE_SYS_SELECT_H */
 #include <stdio.h>
-#ifdef STDC_HEADERS
-# include <stdlib.h>
-# include <stddef.h>
-#else
-# ifdef HAVE_STDLIB_H
-#  include <stdlib.h>
-# endif
-#endif /* STDC_HEADERS */
+#include <stdlib.h>
 #ifdef HAVE_STRING_H
-# if defined(HAVE_MEMORY_H) && !defined(STDC_HEADERS)
-#  include <memory.h>
-# endif
 # include <string.h>
 #endif /* HAVE_STRING_H */
 #ifdef HAVE_STRINGS_H
 # include <strings.h>
 #endif /* HAVE_STRINGS_H */
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif /* HAVE_UNISTD_H */
+#include <unistd.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <limits.h>
@@ -59,12 +40,14 @@ add_preserved_fd(struct preserved_fd_list *pfds, int fd)
     struct preserved_fd *pfd, *pfd_new;
     debug_decl(add_preserved_fd, SUDO_DEBUG_UTIL)
 
-    pfd_new = sudo_emalloc(sizeof(*pfd));
+    pfd_new = malloc(sizeof(*pfd));
+    if (pfd_new == NULL)
+	sudo_fatalx(U_("%s: %s"), __func__, U_("unable to allocate memory"));
     pfd_new->lowfd = fd;
     pfd_new->highfd = fd;
     pfd_new->flags = fcntl(fd, F_GETFD);
     if (pfd_new->flags == -1) {
-	sudo_efree(pfd_new);
+	free(pfd_new);
 	debug_return_int(-1);
     }
 
@@ -73,7 +56,7 @@ add_preserved_fd(struct preserved_fd_list *pfds, int fd)
 	    /* already preserved */
 	    sudo_debug_printf(SUDO_DEBUG_DEBUG|SUDO_DEBUG_LINENO,
 		"fd %d already preserved", fd);
-	    sudo_efree(pfd_new);
+	    free(pfd_new);
 	    break;
 	}
 	if (fd < pfd->highfd) {
@@ -101,7 +84,7 @@ closefrom_except(int startfd, struct preserved_fd_list *pfds)
 {
     int fd, lastfd = -1;
     struct preserved_fd *pfd, *pfd_next;
-    fd_set *fdsp;
+    unsigned char *fdbits;
     debug_decl(closefrom_except, SUDO_DEBUG_UTIL)
 
     /* First, relocate preserved fds to be as contiguous as possible.  */
@@ -140,16 +123,18 @@ closefrom_except(int startfd, struct preserved_fd_list *pfds)
     }
 
     /* Create bitmap of preserved (relocated) fds.  */
-    fdsp = sudo_ecalloc(howmany(lastfd + 1, NFDBITS), sizeof(fd_mask));
+    fdbits = calloc((lastfd + NBBY) / NBBY, 1);
+    if (fdbits == NULL)
+	sudo_fatalx(U_("%s: %s"), __func__, U_("unable to allocate memory"));
     TAILQ_FOREACH(pfd, pfds, entries) {
-	FD_SET(pfd->lowfd, fdsp);
+	sudo_setbit(fdbits, pfd->lowfd);
     }
 
     /*
      * Close any unpreserved fds [startfd,lastfd]
      */
     for (fd = startfd; fd <= lastfd; fd++) {
-	if (!FD_ISSET(fd, fdsp)) {
+	if (!sudo_isset(fdbits, fd)) {
 	    sudo_debug_printf(SUDO_DEBUG_DEBUG|SUDO_DEBUG_LINENO,
 		"closing fd %d", fd);
 #ifdef __APPLE__
@@ -160,7 +145,7 @@ closefrom_except(int startfd, struct preserved_fd_list *pfds)
 #endif
 	}
     }
-    free(fdsp);
+    free(fdbits);
 
     /* Let closefrom() do the rest for us. */
     if (lastfd + 1 > startfd)
