@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1993-1996, 1998-2005, 2007-2013
+ * Copyright (c) 1993-1996, 1998-2005, 2007-2014
  *	Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -19,8 +19,8 @@
  * Materiel Command, USAF, under agreement number F39502-99-1-0512.
  */
 
-#ifndef _SUDO_SUDO_H
-#define _SUDO_SUDO_H
+#ifndef SUDO_SUDO_H
+#define SUDO_SUDO_H
 
 #include <limits.h>
 #include <pathnames.h>
@@ -33,7 +33,6 @@
 #include "sudo_gettext.h"	/* must be included before sudo_compat.h */
 
 #include "sudo_compat.h"
-#include "sudo_alloc.h"
 #include "sudo_fatal.h"
 #include "sudo_conf.h"
 #include "sudo_debug.h"
@@ -85,6 +84,12 @@
 #define TGP_MASK	0x08		/* mask user input when reading */
 #define TGP_NOECHO_TRY	0x10		/* turn off echo if possible */
 
+/* name/value pairs for command line settings. */
+struct sudo_settings {
+    const char *name;
+    const char *value;
+};
+
 struct user_details {
     pid_t pid;
     pid_t ppid;
@@ -178,7 +183,6 @@ void cleanup(int);
 
 /* tgetpass.c */
 char *tgetpass(const char *, int, int);
-int tty_present(void);
 
 /* exec.c */
 int pipe_nonblock(int fds[2]);
@@ -186,11 +190,11 @@ int sudo_execute(struct command_details *details, struct command_status *cstat);
 
 /* parse_args.c */
 int parse_args(int argc, char **argv, int *nargc, char ***nargv,
-    char ***settingsp, char ***env_addp);
+    struct sudo_settings **settingsp, char ***env_addp);
 extern int tgetpass_flags;
 
 /* get_pty.c */
-int get_pty(int *master, int *slave, char *name, size_t namesz, uid_t uid);
+bool get_pty(int *master, int *slave, char *name, size_t namesz, uid_t uid);
 
 /* sudo.c */
 bool exec_setup(struct command_details *details, const char *ptyname, int ptyfd);
@@ -199,6 +203,7 @@ int run_command(struct command_details *details);
 int os_init_common(int argc, char *argv[], char *envp[]);
 extern const char *list_user;
 extern struct user_details user_details;
+extern int sudo_debug_instance;
 
 /* sudo_edit.c */
 int sudo_edit(struct command_details *details);
@@ -237,7 +242,7 @@ char *getenv_unhooked(const char *name);
 int get_net_ifs(char **addrinfo);
 
 /* ttyname.c */
-char *get_process_ttyname(void);
+char *get_process_ttyname(char *name, size_t namelen);
 
 /* signal.c */
 struct sigaction;
@@ -255,4 +260,4 @@ int add_preserved_fd(struct preserved_fd_list *pfds, int fd);
 void closefrom_except(int startfd, struct preserved_fd_list *pfds);
 void parse_preserved_fds(struct preserved_fd_list *pfds, const char *fdstr);
 
-#endif /* _SUDO_SUDO_H */
+#endif /* SUDO_SUDO_H */
